from web3 import Web3

class AddressEthUniswap:
    uniswap_v3_quoter_address=Web3.to_checksum_address('0xb27308f9F90D607463bb33eA1BeBb41C27CE5AB6')
    uniswap_v3_quoter_v2_address=Web3.to_checksum_address('0x61fFE014bA17989E743c5F6cB21bF9697530B21e')
    uniswap_v3_router_address=Web3.to_checksum_address('0xE592427A0AEce92De3Edee1F18E0157C05861564')
    uniswap_v3_router_v2_address=Web3.to_checksum_address('0x68b3465833fb72A70ecDF485E0e4C7bD8665Fc45')
    uniswap_v3_factory_address=Web3.to_checksum_address('0x1F98431c8aD98523631AE4a59f267346ea31F984')

class AddressEthTokens:
    dai=Web3.to_checksum_address('0x6B175474E89094C44Da98b954EedeAC495271d0F')
    usdc=Web3.to_checksum_address('0xA0b86991c6218b36c1d19D4a2e9Eb0cE3606eB48')
    usdt=Web3.to_checksum_address('0xdAC17F958D2ee523a2206206994597C13D831ec7')
    weth=Web3.to_checksum_address('0xC02aaA39b223FE8D0A0e5C4F27eAD9083C756Cc2')
    wbtc=Web3.to_checksum_address('0x2260FAC5E5542a773Aa44fBCfeDf7C193bc2C599')


name = "dex"

from dexhub.bridge import(
    Synapse
)

from dexhub.dex import(
    Erc20,
    UniswapV2,UniswapV2Pair,
    UniswapV3,UniswapV3Quoter,UniswapV3Factory,UniswapV3Router,
    JoeV2Factory,JoeV2Pair,JoeV2Route,JoeV2Quote,JoeV2Erc20
)

from dexhub.util.helper import(
    DexHelper
)

from dexhub.util.joe_v2 import(
    JoeV2Helper
)

from dexhub.interface.joe import(
    JoeV2Erc20,JoeV2Factory,JoeV2Pair,JoeV2Quote,JoeV2Route
)

from dexhub.address.avax_address import(
    AddressJoe,AddressAvaxTokens,AddressPangolin
)

from dexhub.address.dfk_address import(
    CV,DfkTokens,AddressCV,AddressSynapseCV
)

from dexhub.address.klay_address import(
    SD,KlayToken,AddressClaySwap,AddressSD,AddressSynapseKlay,AddressKlaySwap
)

from dexhub.address.polygon_address import(
    AddressUniswapV3Polygon,PolyTokens
)

from dexhub.address.arbitrum_address import(
    AddressArbitrumJoe,AddressArbitrumTokens,AddressAbitrumSushi
)

from dexhub.address.eth_address import(
    AddressEthUniswap,AddressEthTokens
)

from dexhub.abi.dfk_abi import(
    AbiCv,AbiSynapse
)

from dexhub.abi.klay_abi import(
    AbiClaySwap,AbiSd,AbiSynapseKlay
)

from dexhub.abi.eth_abi import(
    AbiUniswapV3
)

from dexhub.abi.avax_abi import(
    AbiJoe,AbiPangolin
)

from dexhub.abi.arbitrum_abi import(
    AbiArbitrumJoe,AbiArbitrumSushi
)


from dexhub.address.klay_address import SD,KlayToken,AddressClaySwap,AddressSD,AddressSynapseKlay,AddressKlaySwap
from dexhub.address.dfk_address import CV,DfkTokens,AddressCV,AddressSynapseCV
from dexhub.address.polygon_address import AddressUniswapV3Polygon,PolyTokens
from dexhub.address.avax_address import AddressJoe,AddressPangolin,AddressAvaxTokens
from dexhub.address.arbitrum_address import AddressAbitrumSushi,AddressArbitrumJoe,AddressArbitrumTokens
from dexhub.address.eth_address import AddressEthUniswap,AddressEthTokens


from web3 import Web3

class Synapse:

    chain_id_dfk=53935
    chain_id_klay=8217


    def depositETH(_web3:Web3,_contract_address,_contract_abi,_to_chain_id,_amount,_public_key,_private_key):
        l1_bridge_zap_contract=_web3.eth.contract(address=_contract_address, abi=_contract_abi)
        nonce=_web3.eth.get_transaction_count(_public_key)
        deposit_ETH_function=l1_bridge_zap_contract.functions.depositETH(_public_key,_to_chain_id,_amount)
        tx_params={
            'from': _public_key,
            'value': _amount,
            'nonce': nonce,
            'type': '0x2'
        }
        transaction=deposit_ETH_function.build_transaction(tx_params)
        signed_transaction=_web3.eth.account.sign_transaction(transaction, private_key=_private_key)
        return _web3.to_hex(_web3.eth.send_raw_transaction(signed_transaction.rawTransaction))
    
    def redeem(_web3:Web3,_contract_address,_contract_abi,_to_chain_id,_token,_amount,_public_key,_private_key):
        bridge_zap_contract=_web3.eth.contract(address=_contract_address, abi=_contract_abi)
        nonce=_web3.eth.get_transaction_count(_public_key)
        redeem_function=bridge_zap_contract.functions.redeem(_public_key,_to_chain_id,_token,_amount)
        tx_params={
            'from': _public_key,
            'value': 0,
            'nonce': nonce,
            'type': '0x2'
        }
        transaction=redeem_function.build_transaction(tx_params)
        signed_transaction=_web3.eth.account.sign_transaction(transaction, private_key=_private_key)
        return _web3.to_hex(_web3.eth.send_raw_transaction(signed_transaction.rawTransaction))


from web3 import Web3

class JoeV2Helper:
    #获得当前x兑换y的价格
    def get_price(_w3:Web3,_address_pair,_abi_pair,_decimal_x,_decimal_y):
        bin_step=_w3.eth.contract(address=_address_pair,abi=_abi_pair).functions.getBinStep().call()
        s=bin_step/10000
        id=_w3.eth.contract(address=_address_pair,abi=_abi_pair).functions.getActiveId().call()
        price=pow((1+s),id-pow(2,23))*pow(10,_decimal_x)/pow(10,_decimal_y)
        return price



import math
import itertools as it
from requests import HTTPError
from requests import ReadTimeout
from web3 import exceptions
import time
import concurrent.futures
from web3 import Web3
import datetime

from dexhub.dex import UniswapV2
#useful tool for dex
class DexHelper:

    ################################################
    #   optimal trade volume
    ################################################
    #   This is a implement of optimal trade volume,
    #   please reffer to 
    #       https://arxiv.org/pdf/2105.02784.pdf  
    #   for more math detail,
    ################################################

    #uniswap swap fee
    r1=0.997
    r2=1

    #update swap fee based on the protocol
    def set_r1(self,_r1):
        self.r1=_r1

    def set_r2(self,_r2):
        self.r2=_r2

    #trianglular arbitrage optimal volume
    # arbitrage route 1->2->3->1
    # a12 is the token1 reserve volume in token pair 1-2
    def optimal_volume_triangle(self,_a12,_a21,_a23,_a32,_a31,_a13):
        a13_prime=self.get_a13_prime(_a12,_a21,_a23)
        a31_prime=self.get_a31_prime(_a21,_a32,_a23)
        a=self.get_a(a13_prime,_a31,a31_prime)
        a_prime=self.get_a_prime(_a13,a31_prime,_a31)
        denominator=(1+math.isqrt(int(self.r1*self.r2*a_prime*a-1))-a)
        return denominator/self.r1

    #trianglular arbitrage optimal volume
    # arbitrage route 1->2->3->4->1
    # a12 is the token1 reserve volume in token pair 1-2
    def optimal_volume_quadrilateral(self,_a12,_a21,_a23,_a32,_a34,_a43,_a41,_a14):
        #convert _a12 _a21 _a23 _a32 to _a13 _a31
        _a13=self.get_a13_prime(_a12,_a21,_a23)
        _a31=self.get_a31_prime(_a21,_a32,_a23)
        #1341 volume calculate        
        return self.optimal_volume_triangle(_a13,_a31,_a34,_a43,_a41,_a14)


    # general optiimal volume function
    # artbitrage route 1->2->3->...->n->1
    def optimal_volume(self,reserves:list):
        list_length=len(reserves)
        assert(list_length%2==0)
        assert(list_length>=6)
        if list_length>6:
            for i in range(int(list_length/2-3)):
                _a13=self.get_a13_prime(reserves[0],reserves[1],reserves[2])
                _a31=self.get_a31_prime(reserves[1],reserves[3],reserves[2])
                del reserves[3]
                del reserves[2]
                reserves[0]=_a13
                reserves[1]=_a31
        assert(len(reserves)==6)
        return self.optimal_volume_triangle(reserves[0],reserves[1],reserves[2],reserves[3],reserves[4],reserves[5])

    def get_a(self,_a13_prime,_a31,_a31_prime):
        return int(_a13_prime*_a31/(_a31+self.r1*self.r2*_a31_prime))

    def get_a_prime(self,_a13,_a31_prime,_a31):
        return int((self.r1*self.r2*_a13*_a31_prime)/(_a31+self.r1*self.r2*_a31_prime))
    
    def get_a13_prime(self,_a12,_a21,_a23):
        return int(_a12*_a23/(_a23+self.r1*self.r2*_a21))
        
    def get_a31_prime(self,_a21,_a32,_a23):
        return int(self.r1*self.r2*_a21*_a32/(_a23+self.r1*self.r2*_a21))


    ################################################
    #   prepair the pair list and path list 
    #   for arbitrage bot
    ################################################
    #
    #
    ################################################

    def get_pair_list_and_path_list(self,_arbitrage_token:str,_token_list:list,_dex:UniswapV2):
        # save the pair tokens address with reserve
        pair_list=[]
        # save the path which is available to arbitrage
        path_list=[]

        #get pair list
        #pair_list=[token0_address,token1_address,pair_address]
        for token in _token_list:
            pair_address=_dex.get_pair(_arbitrage_token,token)
            if (pair_address!="0x0000000000000000000000000000000000000000"):
                token0Address,reserve0,token1Address,reserve1,blockTimeStamp,price0,price1=_dex.get_pair_info(pair_address)
                pair_list.append([token0Address,token1Address,pair_address])
        pair_combination=it.combinations(_token_list,2)
        for pair in pair_combination:
            #make sure all the pair exist
            pair_address=_dex.get_pair(pair[0],pair[1])
            if (pair_address!="0x0000000000000000000000000000000000000000"):
                token0Address,reserve0,token1Address,reserve1,blockTimeStamp,price0,price1=_dex.get_pair_info(pair_address)
                pair_list.append([token0Address,token1Address,pair_address])

        #get path list
        pair_permutation=it.permutations(_token_list,2)
        for pair in pair_permutation:
            flag_pair0_pair1=_dex.get_pair(pair[0],pair[1])!="0x0000000000000000000000000000000000000000"
            flag_pair0_token=_dex.get_pair(pair[0],_arbitrage_token)!="0x0000000000000000000000000000000000000000"
            flag_pair1_token=_dex.get_pair(pair[1],_arbitrage_token)!="0x0000000000000000000000000000000000000000"
            if (flag_pair0_pair1 and flag_pair0_token and flag_pair1_token):
                path_list.append([_arbitrage_token,pair[0],pair[1],_arbitrage_token])
        tri_pair_permutation=it.permutations(_token_list,3)
        for pair in tri_pair_permutation:
            flag_pair0_pair1=_dex.get_pair(pair[0],pair[1]) !="0x0000000000000000000000000000000000000000"
            flag_pair1_pair2=_dex.get_pair(pair[1],pair[2]) !="0x0000000000000000000000000000000000000000"
            flag_pair0_token=_dex.get_pair(pair[0],_arbitrage_token) !="0x0000000000000000000000000000000000000000"
            flag_pair2_token=_dex.get_pair(pair[2],_arbitrage_token) !="0x0000000000000000000000000000000000000000"
            if ( flag_pair0_pair1 and flag_pair1_pair2 and flag_pair0_token and flag_pair2_token):
                path_list.append([_arbitrage_token,pair[0],pair[1],pair[2],_arbitrage_token])
        return pair_list,path_list
    
    def get_pair_map_and_path_list(_arbitrage_token:str,_token_list:list,_dex:UniswapV2):
        pair_reserve_dict={}
        path_list=[]

        #get pair dict
        #pair_dict=[token0_address,reserve0,reserve1,token1_address,pair_address]
        for token in _token_list:
            pair_address=_dex.get_pair(_arbitrage_token,token)
            if (pair_address!="0x0000000000000000000000000000000000000000"):
                token0Address,reserve0,token1Address,reserve1,blockTimeStamp,price0,price1=_dex.get_pair_info(pair_address)
                key=token0Address+token1Address
                pair_reserve_dict[str(key)]=[pair_address,reserve0,reserve1,token0Address,token1Address]
        pair_combination=it.combinations(_token_list,2)
        for pair in pair_combination:
            #make sure all the pair exist
            pair_address=_dex.get_pair(pair[0],pair[1])
            if (pair_address!="0x0000000000000000000000000000000000000000"):
                token0Address,reserve0,token1Address,reserve1,blockTimeStamp,price0,price1=_dex.get_pair_info(pair_address)
                key=token0Address+token1Address
                pair_reserve_dict[str(key)]=[pair_address,reserve0,reserve1,token0Address,token1Address]

        #get path list
        pair_permutation=it.permutations(_token_list,2)
        for pair in pair_permutation:
            flag_pair0_pair1=DexHelper.check_pair(pair[0],pair[1],pair_reserve_dict)
            flag_pair0_token=DexHelper.check_pair(pair[0],_arbitrage_token,pair_reserve_dict)
            flag_pair1_token=DexHelper.check_pair(pair[1],_arbitrage_token,pair_reserve_dict)
            if (flag_pair0_pair1 and flag_pair0_token and flag_pair1_token):
                path_list.append([_arbitrage_token,pair[0],pair[1],_arbitrage_token])
        tri_pair_permutation=it.permutations(_token_list,3)
        for pair in tri_pair_permutation:
            flag_pair0_pair1=DexHelper.check_pair(pair[0],pair[1],pair_reserve_dict)
            flag_pair1_pair2=DexHelper.check_pair(pair[1],pair[2],pair_reserve_dict)
            flag_pair0_token=DexHelper.check_pair(pair[0],_arbitrage_token,pair_reserve_dict)
            flag_pair2_token=DexHelper.check_pair(pair[2],_arbitrage_token,pair_reserve_dict)
            if ( flag_pair0_pair1 and flag_pair1_pair2 and flag_pair0_token and flag_pair2_token):
                path_list.append([_arbitrage_token,pair[0],pair[1],pair[2],_arbitrage_token])
        return pair_reserve_dict,path_list
    
    def check_pair(_token0:str,_token1:str,_pair_reserve_dict:dict):
        value=None
        try:
            key=_token0+_token1
            value=_pair_reserve_dict[key]
        except KeyError:
            pass
        try:
            key=_token1+_token0
            value=_pair_reserve_dict[key]
        except KeyError:
            pass
        return value is not None

    ################################################
    #   get token decimal information
    #   
    ################################################
    #
    #
    ################################################
    def get_token_decimal(_token_list:list,_web3:Web3):
        token_decimal_dict={}
        for token in _token_list:
            token_contract=_web3.eth.contract(address=token, abi=UniswapV2.uniswap_v2_erc20_abi)
            decimal=token_contract.functions.decimals().call()
            token_decimal_dict[token]=decimal
        return token_decimal_dict

    ################################################
    #   get reserve information
    #   
    ################################################
    #
    #
    ################################################  
    def get_pair_reserve_dict(self,_pair_list:list,_dex:UniswapV2):
        # save the token0token1   ----    reserve0,reserve1 dictionary
        pair_reserve_dict={}
        for pair in _pair_list:
            pair_address=pair[2]
            reserve0,reserve1,blockTimeStamp=_dex.get_pair_reserve(pair_address)
            key=pair[0]+pair[1]
            pair_reserve_dict[str(key)]=[pair_address,reserve0,reserve1]
        return pair_reserve_dict

    def get_pair_reserve_dict_with_decimal(self,_pair_list:list,_dex:UniswapV2,token_decimal:dict):
        # save the token0token1   ----    reserve0,reserve1 dictionary
        pair_reserve_dict={}
        for pair in _pair_list:
            pair_address=pair[2]
            reserve0,reserve1,blockTimeStamp=_dex.get_pair_reserve(pair_address)
            key=pair[0]+pair[1]
            pair_reserve_dict[str(key)]=[pair_address,reserve0*pow(10,18-token_decimal[pair[0]]),reserve1*pow(10,18-token_decimal[pair[1]])]
        return pair_reserve_dict

    def update_pair_reserve_dict(_pair_reserve_dict:dict,_dex:UniswapV2):
        for key in _pair_reserve_dict:
            value=_pair_reserve_dict[key]
            pair_address=value[0]
            reserve0,reserve1,blockTimeStamp=_dex.get_pair_reserve(pair_address)
            value[1]=reserve0
            value[2]=reserve1
            _pair_reserve_dict[key]=value

    def update_pair_reserve_dict_with_decimal(_pair_reserve_dict:dict,_dex:UniswapV2,token_decimal:dict):
        for key in _pair_reserve_dict:
            value=_pair_reserve_dict[key]
            pair_address=value[0]
            reserve0,reserve1,blockTimeStamp=_dex.get_pair_reserve(pair_address)
            value[1]=reserve0*pow(10,18-token_decimal[value[3]])
            value[2]=reserve1*pow(10,18-token_decimal[value[4]])
            _pair_reserve_dict[key]=value

    def update_pair_reserve_dict_concurrent(_pair_reserve_dict:dict,_dex:UniswapV2):
        with concurrent.futures.ThreadPoolExecutor(max_workers=100) as executor:
            for key in _pair_reserve_dict:
                executor.submit(DexHelper.get_reserve,_dex,key,_pair_reserve_dict)

    def get_reserve(_dex:UniswapV2,_key:str,_pair_reserve_dict):
            print('start========='+str(datetime.datetime.now()))
            value=_pair_reserve_dict[_key]
            pair_address=value[0]
            reserve0,reserve1,blockTimeStamp=_dex.get_pair_reserve(pair_address)
            value[1]=reserve0
            value[2]=reserve1
            _pair_reserve_dict[_key]=value
            print('end========='+str(datetime.datetime.now()))

    #throw except if fail to find the reserve which should not happen
    #return list of reserves in order of a12 a21 a23 a32 ... a(n-1)(n)  a(n)(n-1)   a(n)(1)  a(1)(n)    
    def path_to_reserve(self,_path:list,_pair_reserve_dict):
        reserve_list=[]
        for i in range(len(_path)-1):
            try:
                address,a01,a10=_pair_reserve_dict[(_path[i]+_path[i+1])]
            except KeyError as k:
                address,a10,a01=_pair_reserve_dict[(_path[i+1]+_path[i])]
            assert(a01>50000)
            assert(a10>50000)
            reserve_list.append(a01)
            reserve_list.append(a10)
        return reserve_list

    #return int, if result >0, there is arbitrage chance
    def get_arbitrage_space(self,_reserve_list:list,_ex_rate):
        assert(len(_reserve_list)%2==0)
        p=1
        for i in range(int(len(_reserve_list)/2)):
            p=p*(_reserve_list[i*2+1]/_reserve_list[i*2])*_ex_rate
        return p-1


    #arbitrage swap function
    def swap(self,_dex:UniswapV2,_amountIn,_amountOut,_path:list,_to_address,_deadline,_gas):
        try:
            _dex.swapExactTokenForTokensWithGasWithNonce(_amountIn,_amountOut,_path,_to_address,_deadline,600000,_dex.nonce)
            _dex.nonce+=1
        except exceptions.ContractLogicError as e:
            print(e)
        except exceptions.ValidationError as e:
            print(e)
        except ReadTimeout as e:
            print(e)
        except HTTPError as e:
            print(e)
        except ValueError as e:
            print(e)
        except TypeError as e:
            print(e)
        except:
            print("####################################")
            print("################error###############")
            print("####################################")
            time.sleep(2)


class SpatialHelper:
    #uniswap v2 <-> uniswap v2 spatial
    def v2_spatial():
        print('test')


from web3 import Web3

class CV:
    #dex
    cv_route=Web3.to_checksum_address('0x3C351E1afdd1b1BC44e931E12D4E05D6125eaeCa')
    cv_factory=Web3.to_checksum_address('0x794C07912474351b3134E6D6B3B7b3b4A07cbAAa')

class AddressCV:
    #dex
    cv_route=Web3.to_checksum_address('0x3C351E1afdd1b1BC44e931E12D4E05D6125eaeCa')
    cv_factory=Web3.to_checksum_address('0x794C07912474351b3134E6D6B3B7b3b4A07cbAAa')

class AddressSynapseCV:
    cv_l1_bridge_zap=Web3.to_checksum_address('0x75224b0f245Fe51d5bf47A898DbB6720D4150BA7')

class DfkTokens:
    #erc20
    crystal=Web3.to_checksum_address("0x04b9dA42306B023f3572e106B11D82aAd9D32EBb")
    xjewel=Web3.to_checksum_address("0x77f2656d04E158f915bC22f07B779D94c1DC47Ff")
    wjewel=Web3.to_checksum_address("0xCCb93dABD71c8Dad03Fc4CE5559dC3D89F67a260")
    avax=Web3.to_checksum_address("0xB57B60DeBDB0b8172bb6316a9164bd3C695F133a")
    usdc=Web3.to_checksum_address("0x3AD9DFE640E1A9Cc1D9B0948620820D975c3803a")
    eth=Web3.to_checksum_address("0xfBDF0E31808d0aa7b9509AA6aBC9754E48C58852")
    btc=Web3.to_checksum_address("0x7516EB8B8Edfa420f540a162335eACF3ea05a247")
    klay=Web3.to_checksum_address("0x97855Ba65aa7ed2F65Ed832a776537268158B78a")
    matic=Web3.to_checksum_address("0xD17a41Cd199edF1093A9Be4404EaDe52Ec19698e")
    ftm=Web3.to_checksum_address("0x2Df041186C844F8a2e2b63F16145Bc6Ff7d23E25")
    gold=Web3.to_checksum_address("0x576C260513204392F0eC0bc865450872025CB1cA")
    tears=Web3.to_checksum_address("0x79fE1fCF16Cc0F7E28b4d7B97387452E3084b6dA")



    Ambertaffy="0xB78d5580d6D897DE60E1A942A5C1dc07Bc716943"
    Milkweed="0xA2cef1763e59198025259d76Ce8F9E60d27B17B5"
    Darkweed="0x848Ac8ddC199221Be3dD4e4124c462B806B6C4Fd"
    Goldvein="0x0096ffda7A8f8E00e9F8Bbd1cF082c14FA9d642e"
    SpiderFruit="0x3E022D84D397F18743a90155934aBAC421D5FA4C"
    Ragweed="0x137995beEEec688296B0118131C1052546475fF3"
    Redleaf="0x473A41e71618dD0709Ba56518256793371427d79"
    Rockroot="0x60170664b52c035Fcb32CF5c9694b22b47882e5F"
    blue_stem="0x0776b936344DE7bd58A4738306a6c76835ce5D3F"
    Swift_Thistle="0x97b25DE9F61BBBA2aD51F1b706D4D7C04257f33A"
    Frost_Drum="0xe7a1B580942148451E47b92e95aEB8d31B0acA37"
    Knaproot="0xBcdD90034eB73e7Aec2598ea9082d381a285f63b"
    Shaggy_Caps="0x80A42Dc2909C0873294c5E359e8DF49cf21c74E4"
    Skunk_Shade="0xc6030Afa09EDec1fd8e63a1dE10fC00E0146DaF3"

    Bloater="0x268CC8248FFB72Cd5F3e73A9a20Fa2FF40EfbA61"
    Ironscale="0x04B43D632F34ba4D4D72B0Dc2DC4B30402e5Cf88"
    Lanterneye="0xc2Ff93228441Ff4DD904c60Ecbc1CfA2886C76eB"
    Redgill="0x68eE50dD7F1573423EE0Ed9c66Fc1A696f937e81"
    Sailfish="0x7f46E45f6e0361e7B9304f338404DA85CB94E33D"
    Shimmerskin="0xd44ee492889C078934662cfeEc790883DCe245f3"
    Silverfin="0xA7CFd21223151700FB82684Cd9c693596267375D"
    Frost_Bloater="0x3bcb9A3DaB194C6D8D44B424AF383E7Db51C82BD"
    Speckle_Tail="0xE7CB27ad646C49dC1671Cb9207176D864922C431"
    King_Pincer="0x60A3810a3963f23Fa70591435bbe93BF8786E202"
    Three_Eyed_Eel="0x6513757978E89e822772c16B60AE033781A29A4F"


    shvas="0x75E8D8676d774C9429FbB148b30E304b5542aC3d"
    moksha="0xCd2192521BD8e33559b0CA24f3260fE6A26C28e4"


    blue_egg="0xa61Bac689AD6867a605633520D70C49e1dCce853"
    grey_egg="0x7E121418cC5080C96d967cf6A033B0E541935097"
    green_egg="0x8D2bC53106063A37bb3DDFCa8CfC1D262a9BDCeB"
    yellow_egg="0x72F860bF73ffa3FC42B97BbcF43Ae80280CFcdc3"
    golden_egg="0xf2D479DaEdE7F9e270a90615F8b1C52F3C487bC7"


    health_vial="0x591853e01EcFDcF1Bdc9f093423C197BfBBd1A4f"
    full_health_potion="0x5948dd8Df6afEFE05B033AD8f3ae513a9Cd4F1Dc"
    mana_vial="0x240da5314B05E84392e868aC8f2b80ad6becadd4"
    full_mana_potion="0xf17FD21bDF6713a1Dfed668b97835b21e32651e8"
    anti_poison_potion="0x449eB718e351a86718A090A1a8Db3FD561306d9b"
    anti_blind_potion="0x5986045e7c221c8AD40A736B6434D82E29687aeB"
    magic_resistance_potion="0xFADCb72aAE2713975a890b59FF47231D1A552De3"
    toughness_potion="0x2dfFf745d2c7ddCAD4E97b80DF33705B1a95A172"
    stamina_vial="0x242078edFDca25ef2A497C8D9f256Fd641472E5F"
    swiftness_potion="0x84246Ce3988742D46fC00d9b8b2AFb5CDBDaE660"

    l_swiftness_crystall="0x6BCA53314dADdA7f4De30A95413f75a93bfAfecF"
    l_might_crystal="0x5bAC3cAd961B01Ef9510C8e6c5402A2bB1542831"
    l_finness_crystall="0x9d9ef1Bf6A46b8413bf6b1b54F6A7aAb53c6b1b6"
    l_vigor_crystall="0x5e4Cf6907CB5fBe2F642E399F6d07E567155d1F8"
    l_fortitude_crystall="0xbd2677c06C9448534A851bdD25dF045872b87cb1"
    l_wit_crystall="0xC989c916F189D2A2BE0322c020942d7c43aEa830"
    l_insight_crystall="0xbb5F97358F60cCBa262883A3Ff0C637393FE3aB8"
    l_fortune_crystall="0xE410b2BE2Ce1508E15009118567d02C6d7A7038e"
    l_chaos_crystall="0xeEe5b16Cc49e7cef65391Fe7325cea17f787e245"


from web3 import Web3

class AddressUniswapV3Polygon:
    route_v2=Web3.to_checksum_address('0x68b3465833fb72A70ecDF485E0e4C7bD8665Fc45')
    factory=Web3.to_checksum_address('0x1F98431c8aD98523631AE4a59f267346ea31F984')
    quoter_v2=Web3.to_checksum_address('0x61fFE014bA17989E743c5F6cB21bF9697530B21e')



class PolyTokens:
    usdc=Web3.to_checksum_address('0x2791Bca1f2de4661ED88A30C99A7a9449Aa84174')
    usdt=Web3.to_checksum_address('0xc2132D05D31c914a87C6611C10748AEb04B58e8F')
    weth=Web3.to_checksum_address('0x7ceB23fD6bC0adD59E62ac25578270cFf1b9f619')
    wbtc=Web3.to_checksum_address('0x1BFD67037B42Cf73acF2047067bd4F2C47D9BfD6')
    dai=Web3.to_checksum_address('0x8f3Cf7ad23Cd3CaDbD9735AFf958023239c6A063')